@@ -38,7 +38,6 @@
  * LibBalsa includes.
  */
 #include "address-book.h"
-#include "cell-renderer-button.h"
 #include "misc.h"
 
 struct _LibBalsaAddressView {
@@ -54,7 +53,9 @@ struct _LibBalsaAddressView {
 
     gchar *domain;
 
+    GtkTreeViewColumn *button_column;
     GtkTreeViewColumn *type_column;
+    GtkTreeViewColumn *dropdown_column;
     GtkTreeViewColumn *focus_column;
     GtkCellRenderer   *renderer_combo;
 
@@ -159,8 +160,8 @@ const gchar *const libbalsa_address_view_types[] = {
     N_("Reply To:"),
 };
 
-/* Pixbufs */
-static GdkPixbuf *lbav_book_icon, *lbav_close_icon, *lbav_drop_down_icon;
+/* Icon names */
+static const char *lbav_book_icon, *lbav_close_icon, *lbav_drop_down_icon;
 
 /*
  *     Helpers
@@ -271,10 +272,12 @@ lbav_ensure_blank_line_idle_cb(LibBalsaAddressView * address_view)
     gtk_tree_row_reference_free(address_view->focus_row);
     address_view->focus_row = NULL;
 
-    /* This will open the entry for editing */
-    gtk_tree_view_set_cursor(GTK_TREE_VIEW(address_view), focus_path,
-                             address_view->focus_column, TRUE);
-    gtk_tree_path_free(focus_path);
+    if (focus_path != NULL) {
+        /* This will open the entry for editing */
+        gtk_tree_view_set_cursor(GTK_TREE_VIEW(address_view), focus_path,
+                                 address_view->focus_column, TRUE);
+        gtk_tree_path_free(focus_path);
+    }
 
     address_view->focus_idle_id = 0;
 
@@ -590,12 +593,13 @@ lbav_entry_changed_cb(GtkEntry * entry, LibBalsaAddressView * address_view)
  */
 static gboolean
 lbav_key_pressed_cb(GtkEntry * entry,
-                    GdkEventKey * event,
+                    const GdkEvent * event,
                     LibBalsaAddressView * address_view)
 {
+    guint keyval;
     GtkEntryCompletion *completion;
 
-    if (event->keyval != GDK_KEY_Escape)
+    if (!gdk_event_get_keyval(event, &keyval) || keyval != GDK_KEY_Escape)
         return FALSE;
 
     if (address_view->last_was_escape) {
@@ -678,6 +682,8 @@ lbav_completion_match_selected_cb(GtkEntryCompletion * completion,
     g_signal_handlers_unblock_by_func(entry, lbav_entry_changed_cb,
                                       address_view);
 
+    g_free(name);
+
     return TRUE;
 }
 
@@ -732,24 +738,32 @@ lbav_editing_done(GtkCellEditable * cell_editable,
 
 
 /*
- * Focus Out callback
+ * notify::has-focus callback
  * If only one completion matches, fill it into the entry
  */
-static gboolean
-lbav_focus_out_cb(GtkEntry * entry, GdkEventFocus * event,
-                  LibBalsaAddressView * address_view)
+static void
+lbav_notify_has_focus_cb(GtkEntry            *entry,
+                         GParamSpec          *pspec,
+                         LibBalsaAddressView *address_view)
 {
-    const gchar *the_entry = gtk_entry_get_text(entry);
+    const gchar *the_entry;
 
-    if (the_entry && *the_entry) {
+    if (gtk_widget_has_focus(GTK_WIDGET(entry))) {
+        /* Not a focus-out event */
+        return;
+     }
+
+    the_entry = gtk_entry_get_text(entry);
+
+    if (the_entry != NULL && the_entry[0] != '\0') {
         GList *match;
 
         match = lbav_get_matching_addresses(address_view,
                                             the_entry,
                                             LIBBALSA_ADDRESS_VIEW_MATCH_ALL);
 
-        if (match) {
-            if (!match->next) {
+        if (match != NULL) {
+            if (match->next == NULL) {
                 gchar *the_addr =
                     internet_address_to_string((InternetAddress *) match->data,
                                                 NULL, FALSE);
@@ -767,8 +781,6 @@ lbav_focus_out_cb(GtkEntry * entry, GdkEventFocus * event,
             g_list_free_full(match, g_object_unref);
         }
     }
-
-    return FALSE;
 }
 
 
@@ -810,8 +822,8 @@ lbav_row_editing_cb(GtkCellRenderer * renderer,
                      G_CALLBACK(lbav_insert_text_cb), address_view);
     g_signal_connect(editable, "editing-done",
                      G_CALLBACK(lbav_editing_done), address_view);
-    g_signal_connect_after(GTK_ENTRY(editable), "focus-out-event",
-			   G_CALLBACK(lbav_focus_out_cb), address_view);
+    g_signal_connect_after(editable, "notify::has-focus",
+                     G_CALLBACK(lbav_notify_has_focus_cb), address_view);
     gtk_entry_set_completion(GTK_ENTRY(editable), completion);
     g_object_unref(completion);
 
@@ -822,28 +834,24 @@ lbav_row_editing_cb(GtkCellRenderer * renderer,
 }
 
 /*
- *     Callback for the button's "activated" signal
+ *     A button was clicked
  */
 static void
-lbav_button_activated_cb(LibBalsaCellRendererButton * button,
-                         const gchar * path_string,
-                         LibBalsaAddressView * address_view)
+lbav_button_activated(LibBalsaAddressView *address_view,
+                      GtkTreePath         *path)
 {
     GtkTreeView *tree_view = GTK_TREE_VIEW(address_view);
     GtkTreeModel *model = gtk_tree_view_get_model(tree_view);
-    GtkTreePath *path;
     GtkTreeIter iter;
-    GdkPixbuf *pixbuf;
+    char *icon_name;
 
-    path = gtk_tree_path_new_from_string(path_string);
     if (!gtk_tree_model_get_iter(model, &iter, path)) {
-        gtk_tree_path_free(path);
         return;
     }
 
-    gtk_tree_model_get(model, &iter, ADDRESS_ICON_COL, &pixbuf, -1);
+    gtk_tree_model_get(model, &iter, ADDRESS_ICON_COL, &icon_name, -1);
 
-    if (pixbuf == lbav_close_icon) {
+    if (strcmp(icon_name, lbav_close_icon) == 0) {
         /* User clicked a remove button. */
         GtkListStore *address_store = GTK_LIST_STORE(model);
         guint type;
@@ -863,29 +871,41 @@ lbav_button_activated_cb(LibBalsaCellRendererButton * button,
         gtk_tree_row_reference_free(row_ref);
     }
 
-    g_object_unref(pixbuf);
-    gtk_tree_path_free(path);
+    g_free(icon_name);
 }
 
 /*
- *     Callback for the drop_down's "activated" signal
- *
  *     Pop up the address type combo-box
  */
 static void
-lbav_drop_down_activated_cb(LibBalsaCellRendererButton * drop_down,
-                            const gchar * path_string,
-                            LibBalsaAddressView * address_view)
+lbav_dropdown_activated(LibBalsaAddressView *address_view,
+                        GtkTreePath         *path)
 {
-    GtkTreePath *path;
-
-    path = gtk_tree_path_new_from_string(path_string);
     gtk_tree_view_set_cursor_on_cell(GTK_TREE_VIEW(address_view),
                                      path,
                                      address_view->type_column,
                                      address_view->renderer_combo,
                                      TRUE);
-    gtk_tree_path_free(path);
+}
+
+/*
+ *     Callback for the address-view's "row-activated" signal
+ *
+ *     Depending on the clicked column, call either the button's handler
+ *     or the drop-down's handler
+ */
+static void
+lbav_row_activated_cb(GtkTreeView       *tree_view,
+                      GtkTreePath       *path,
+                      GtkTreeViewColumn *column,
+                      gpointer           user_data)
+{
+    LibBalsaAddressView *address_view = LIBBALSA_ADDRESS_VIEW(tree_view);
+
+    if (column == address_view->button_column)
+        lbav_button_activated(address_view, path);
+    else if (column == address_view->dropdown_column)
+        lbav_dropdown_activated(address_view, path);
 }
 
 /*
@@ -921,42 +941,6 @@ lbav_sort_func(GtkTreeModel * model, GtkTreeIter * a, GtkTreeIter * b,
     return is_blank_a - is_blank_b;
 }
 
-/*
- * Callback for the tree-view's selection "changed" signal.
- * Implement one-click actions.
- */
-static void
-lbav_selection_changed_cb(GtkTreeSelection * selection,
-                          LibBalsaAddressView * address_view)
-{
-    GdkEvent *event = gtk_get_current_event();
-
-    if (event) {
-        if (event->type == GDK_BUTTON_PRESS) {
-            GdkEventButton *event_button = (GdkEventButton *) event;
-            GtkTreeView *tree_view = (GtkTreeView *) address_view;
-
-            if (event_button->window ==
-                gtk_tree_view_get_bin_window(tree_view)) {
-                gint x, y;
-                GtkTreePath *path;
-                GtkTreeViewColumn *column;
-
-                gtk_tree_view_convert_widget_to_bin_window_coords
-                    (tree_view, (gint) event_button->x,
-                     (gint) event_button->y, &x, &y);
-
-                if (gtk_tree_view_get_path_at_pos
-                    (tree_view, x, y, &path, &column, NULL, NULL)) {
-                    gtk_tree_view_row_activated(tree_view, path, column);
-                    gtk_tree_path_free(path);
-                }
-            }
-        }
-        gdk_event_free(event);
-    }
-}
-
 /*
  *     Public API.
  */
@@ -974,7 +958,6 @@ libbalsa_address_view_new(const gchar * const *types,
     LibBalsaAddressView *address_view;
     GtkCellRenderer *renderer;
     GtkTreeViewColumn *column;
-    GtkTreeSelection *selection;
 
     /* List store for the widget: */
     address_store = gtk_list_store_new(4,
@@ -985,7 +968,7 @@ libbalsa_address_view_new(const gchar * const *types,
                                        /* ADDRESS_NAME_COL: */
                                        G_TYPE_STRING,
                                        /* ADDRESS_ICON_COL: */
-                                       GDK_TYPE_PIXBUF);
+                                       G_TYPE_STRING);
 
     gtk_tree_sortable_set_default_sort_func(GTK_TREE_SORTABLE
                                             (address_store),
@@ -995,9 +978,11 @@ libbalsa_address_view_new(const gchar * const *types,
          GTK_TREE_SORTABLE_DEFAULT_SORT_COLUMN_ID, GTK_SORT_ASCENDING);
 
     /* The widget: */
-    address_view =
-        g_object_new(LIBBALSA_TYPE_ADDRESS_VIEW, "model", address_store,
-                     "headers-visible", FALSE, NULL);
+    address_view = g_object_new(LIBBALSA_TYPE_ADDRESS_VIEW,
+                                "model", address_store,
+                                "headers-visible", FALSE,
+                                "activate-on-single-click", TRUE,
+                                NULL);
     g_object_unref(address_store);
 
     address_view->types = types;
@@ -1006,18 +991,15 @@ libbalsa_address_view_new(const gchar * const *types,
     address_view->fallback = fallback;
 
     tree_view = GTK_TREE_VIEW(address_view);
-    selection = gtk_tree_view_get_selection(tree_view);
-    g_signal_connect(selection, "changed",
-                     G_CALLBACK(lbav_selection_changed_cb), address_view);
+
+    g_signal_connect(tree_view, "row-activated", G_CALLBACK(lbav_row_activated_cb), NULL);
 
     /* The button: */
-    column = gtk_tree_view_column_new();
-    renderer = libbalsa_cell_renderer_button_new();
-    g_signal_connect(renderer, "activated",
-                     G_CALLBACK(lbav_button_activated_cb), address_view);
+    address_view->button_column = column = gtk_tree_view_column_new();
+    renderer = gtk_cell_renderer_pixbuf_new();
     gtk_tree_view_column_pack_start(column, renderer, FALSE);
     gtk_tree_view_column_set_attributes(column, renderer,
-                                        "pixbuf", ADDRESS_ICON_COL,
+                                        "icon-name", ADDRESS_ICON_COL,
                                         NULL);
     gtk_tree_view_append_column(tree_view, column);
 
@@ -1054,13 +1036,13 @@ libbalsa_address_view_new(const gchar * const *types,
                                             "text", ADDRESS_TYPESTRING_COL,
                                             NULL);
 
+        gtk_tree_view_append_column(tree_view, column);
+
         /* Add a drop-down icon to indicate that this is in fact a
          * combo: */
-        renderer = libbalsa_cell_renderer_button_new();
-        g_signal_connect(renderer, "activated",
-                         G_CALLBACK(lbav_drop_down_activated_cb),
-                         address_view);
-        g_object_set(renderer, "pixbuf", lbav_drop_down_icon, NULL);
+        address_view->dropdown_column = column = gtk_tree_view_column_new();
+        renderer = gtk_cell_renderer_pixbuf_new();
+        g_object_set(renderer, "icon-name", lbav_drop_down_icon, NULL);
         gtk_tree_view_column_pack_start(column, renderer, FALSE);
 
         gtk_tree_view_append_column(tree_view, column);
@@ -1147,14 +1129,18 @@ libbalsa_address_view_add_to_row(LibBalsaAddressView * address_view,
     GtkTreePath *path;
     GtkTreeIter iter;
     guint type;
-    gboolean valid;
+    gboolean valid = FALSE;
 
     g_return_if_fail(LIBBALSA_IS_ADDRESS_VIEW(address_view));
 
     model = gtk_tree_view_get_model(GTK_TREE_VIEW(address_view));
+
     path = gtk_tree_row_reference_get_path(row_ref);
-    valid = gtk_tree_model_get_iter(model, &iter, path);
-    gtk_tree_path_free(path);
+    if (path != NULL) {
+        valid = gtk_tree_model_get_iter(model, &iter, path);
+        gtk_tree_path_free(path);
+    }
+
     if (!valid)
         return;
 
@@ -1269,25 +1255,25 @@ libbalsa_address_view_get_list(LibBalsaAddressView * address_view,
 }
 
 void
-libbalsa_address_view_set_book_icon(GdkPixbuf * icon)
+libbalsa_address_view_set_book_icon(const char * icon)
 {
-    g_return_if_fail(GDK_IS_PIXBUF(icon));
+    g_return_if_fail(icon != NULL);
 
     lbav_book_icon = icon;
 }
 
 void
-libbalsa_address_view_set_close_icon(GdkPixbuf * icon)
+libbalsa_address_view_set_close_icon(const char * icon)
 {
-    g_return_if_fail(GDK_IS_PIXBUF(icon));
+    g_return_if_fail(icon != NULL);
 
     lbav_close_icon = icon;
 }
 
 void
-libbalsa_address_view_set_drop_down_icon(GdkPixbuf * icon)
+libbalsa_address_view_set_drop_down_icon(const char * icon)
 {
-    g_return_if_fail(GDK_IS_PIXBUF(icon));
+    g_return_if_fail(icon != NULL);
 
     lbav_drop_down_icon = icon;
 }
