@@ -118,9 +118,10 @@ static AutocryptRecommend autocrypt_check_ia_list(gpgme_ctx_t           gpgme_ct
 												  time_t                ref_time,
 												  GList               **missing_keys,
 												  GError              **error);
-static gboolean key_button_event_press_cb(GtkWidget      *widget,
-						 				  GdkEventButton *event,
-										  gpointer        data);
+static void row_activated_cb(GtkTreeView       *tree_view,
+                             GtkTreePath       *path,
+                             GtkTreeViewColumn *column,
+                             gpointer           user_data);
 
 
 static sqlite3 *autocrypt_db = NULL;
@@ -454,7 +455,9 @@ autocrypt_db_dialog_run(const gchar *date_string, GtkWindow *parent)
 		G_TYPE_POINTER);											/* key */
 
     tree_view = gtk_tree_view_new_with_model(GTK_TREE_MODEL(model));
-    g_signal_connect(tree_view, "button_press_event", G_CALLBACK(key_button_event_press_cb), dialog);
+
+    g_signal_connect(tree_view, "row-activated", G_CALLBACK(row_activated_cb), dialog);
+
     gtk_container_add(GTK_CONTAINER(scrolled_window), tree_view);
     selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(tree_view));
     gtk_tree_selection_set_mode(selection, GTK_SELECTION_SINGLE);
@@ -667,7 +670,7 @@ extract_ac_keydata(GMimeAutocryptHeader *autocrypt_header, ac_key_data_t *dest,
 	gboolean success = FALSE;
 
 	keydata = g_mime_autocrypt_header_get_keydata(autocrypt_header);
-	if (keydata) {
+	if (keydata != NULL) {
 		gpgme_ctx_t ctx;
 
 		dest->keydata = g_bytes_get_data(keydata, &dest->keysize);
@@ -693,11 +696,14 @@ extract_ac_keydata(GMimeAutocryptHeader *autocrypt_header, ac_key_data_t *dest,
 					if ((key != NULL) && (key->subkeys != NULL)) {
 						dest->fingerprint = g_strdup(key->subkeys->fpr);
 						dest->expires = key->subkeys->expires;
+					} else {
+						success = FALSE;
 					}
 				} else {
 					g_warning("Failed to import or list key data for '%s': %s (%u keys, %u bad)",
 						g_mime_autocrypt_header_get_address_as_string(autocrypt_header),
 						(gpg_error != NULL) ? gpg_error->message : "unknown", (keys != NULL) ? g_list_length(keys) : 0U, bad_keys);
+					success = FALSE;
 				}
 				g_clear_error(&gpg_error);
 
@@ -767,32 +773,18 @@ update_last_seen(GMimeAutocryptHeader *autocrypt_header, GError **error)
 }
 
 
-static gboolean
-key_button_event_press_cb(GtkWidget      *widget,
-						  GdkEventButton *event,
-						  gpointer        data)
+static void
+row_activated_cb(GtkTreeView       *tree_view,
+                 GtkTreePath       *path,
+                 GtkTreeViewColumn *column,
+                 gpointer           user_data)
 {
-    GtkTreeView *tree_view = GTK_TREE_VIEW(widget);
-    GtkTreeSelection *selection = gtk_tree_view_get_selection(tree_view);
-    GtkTreePath *path;
-    GtkTreeIter iter;
     GtkTreeModel *model;
-
-    g_return_val_if_fail(event != NULL, FALSE);
-    if ((event->type != GDK_2BUTTON_PRESS) || event->window != gtk_tree_view_get_bin_window(tree_view)) {
-        return FALSE;
-    }
-
-    if (gtk_tree_view_get_path_at_pos(tree_view, event->x, event->y, &path, NULL, NULL, NULL)) {
-        if (!gtk_tree_selection_path_is_selected(selection, path)) {
-            gtk_tree_view_set_cursor(tree_view, path, NULL, FALSE);
-            gtk_tree_view_scroll_to_cell(tree_view, path, NULL, FALSE, 0, 0);
-        }
-        gtk_tree_path_free(path);
-    }
+    GtkTreeIter iter;
 
     /* note: silently ignore all errors below... */
-    if (gtk_tree_selection_get_selected(selection, &model, &iter)) {
+    model = gtk_tree_view_get_model(tree_view);
+    if (gtk_tree_model_get_iter(model, &iter, path)) {
 		gpgme_ctx_t ctx;
 
 		ctx = libbalsa_gpgme_new_with_proto(GPGME_PROTOCOL_OpenPGP, NULL, NULL, NULL);
@@ -809,9 +801,10 @@ key_button_event_press_cb(GtkWidget      *widget,
 					libbalsa_gpgme_import_bin_key(ctx, key, NULL, NULL) &&
 					libbalsa_gpgme_list_keys(ctx, &keys, NULL, NULL, FALSE, FALSE, TRUE, NULL);
 				if (success && (keys != NULL)) {
+			    	GtkWindow *window = user_data;
 			    	GtkWidget *dialog;
 
-			    	dialog = libbalsa_key_dialog(GTK_WINDOW(data), GTK_BUTTONS_CLOSE, (gpgme_key_t) keys->data, GPG_SUBKEY_CAP_ALL,
+			    	dialog = libbalsa_key_dialog(window, GTK_BUTTONS_CLOSE, (gpgme_key_t) keys->data, GPG_SUBKEY_CAP_ALL,
 			    		NULL, NULL);
 			    	(void) gtk_dialog_run(GTK_DIALOG(dialog));
 			    	gtk_widget_destroy(dialog);
@@ -824,8 +817,6 @@ key_button_event_press_cb(GtkWidget      *widget,
 			gpgme_release(ctx);
 		}
     }
-
-    return TRUE;
 }
 
 #endif  /* ENABLE_AUTOCRYPT */
