@@ -49,6 +49,11 @@
 
 #include <glib/gi18n.h>
 
+#ifdef G_LOG_DOMAIN
+#  undef G_LOG_DOMAIN
+#endif
+#define G_LOG_DOMAIN "send"
+
 typedef struct _MessageQueueItem MessageQueueItem;
 typedef struct _SendMessageInfo SendMessageInfo;
 
@@ -138,34 +143,6 @@ libbalsa_is_sending_mail(void)
 }
 
 
-/* libbalsa_wait_for_sending_thread:
-   wait for the sending thread but not longer than max_time seconds.
-   -1 means wait indefinetely (almost).
- */
-void
-libbalsa_wait_for_sending_thread(gint max_time)
-{
-    gint sleep_time = 0;
-#define DOZE_LENGTH (20 * 1000)
-    static const struct timespec req = {
-        0, DOZE_LENGTH * 1000
-    };   /*nanoseconds*/
-
-    if (max_time < 0) {
-        max_time = G_MAXINT;
-    } else {
-        max_time *= 1000000;  /* convert to microseconds */
-    }
-    while ((g_atomic_int_get(&sending_threads) > 0) && (sleep_time < max_time)) {
-        while (gtk_events_pending()) {
-            gtk_main_iteration_do(FALSE);
-        }
-        nanosleep(&req, NULL);
-        sleep_time += DOZE_LENGTH;
-    }
-}
-
-
 static MessageQueueItem *
 msg_queue_item_new(SendMessageInfo *smi)
 {
@@ -296,7 +273,7 @@ add_mime_body_plain(LibBalsaMessageBody     *body,
         gchar *type, *subtype;
 
         /* FIXME: test sending with different mime types */
-        g_message("path active");
+        g_debug("path active");
         type = g_strdup (body->content_type);
         if ((subtype = strchr (type, '/')) != NULL) {
             *subtype++ = 0;
@@ -401,9 +378,9 @@ static void
 dump_queue(const char *msg)
 {
     MessageQueueItem *mqi = message_queue;
-    printf("dumping message queue at %s:\n", msg);
+    g_debug("dumping message queue at %s:", msg);
     while (mqi) {
-        printf("item: %p\n", mqi);
+        g_debug("item: %p", mqi);
         mqi = mqi->next_message;
     }
 }
@@ -907,7 +884,7 @@ libbalsa_process_queue(LibBalsaMailbox     *outbox,
 			}
 		} else {
 			update_send_timer(TRUE);
-			g_message("%s: no messages pending", __func__);
+			g_debug("%s: no messages pending", __func__);
 		}
 
 		libbalsa_mailbox_close(outbox, FALSE);
@@ -1073,7 +1050,7 @@ balsa_send_message_real(SendMessageInfo *info)
             if (mailbox != NULL) {
                 libbalsa_message_change_flags(mqi->orig, 0, LIBBALSA_MESSAGE_FLAG_FLAGGED);
             } else {
-                g_message("mqi: %p mqi->orig: %p mailbox: %p\n",
+                g_debug("mqi: %p mqi->orig: %p mailbox: %p",
                           mqi, mqi->orig, mailbox);
             }
 
@@ -1200,26 +1177,6 @@ get_mailbox_names(GList               *list,
 }
 
 
-/* We could have used g_strsplit_set(s, "/;", 3) but it is not
- * available in older glib. */
-static gchar **
-parse_content_type(const char *content_type)
-{
-    gchar **ret = g_new0(gchar *, 3);
-    char *delim, *slash = strchr(content_type, '/');
-    if (!slash) {
-        ret[0] = g_strdup(content_type);
-        return ret;
-    }
-    ret[0] = g_strndup(content_type, slash - content_type);
-    slash++;
-    for (delim = slash; *delim && *delim != ';' && *delim != ' '; delim++) {
-    }
-    ret[1] = g_strndup(slash, delim - slash);
-    return ret;
-}
-
-
 static LibBalsaMsgCreateResult
 libbalsa_message_create_mime_message(LibBalsaMessage *message,
                                      gboolean         flow,
@@ -1260,7 +1217,7 @@ libbalsa_message_create_mime_message(LibBalsaMessage *message,
             gchar **mime_type;
 
             if (body->content_type != NULL) {
-                mime_type = parse_content_type(body->content_type);
+                mime_type = g_strsplit_set(body->content_type, "/;", 3);
             } else {
                 const gchar *mt = libbalsa_vfs_get_mime_type(body->file_uri);
                 mime_type = g_strsplit(mt, "/", 2);
@@ -1525,9 +1482,7 @@ libbalsa_message_create_mime_message(LibBalsaMessage *message,
         gchar **pair = list->data;
         g_strchug(pair[1]);
         g_mime_object_append_header(GMIME_OBJECT(mime_message), pair[0], pair[1], NULL);
-#if DEBUG_USER_HEADERS
-        printf("adding header '%s:%s'\n", pair[0], pair[1]);
-#endif
+        g_debug("adding header '%s:%s'", pair[0], pair[1]);
     }
 
     tmp = g_strdup_printf("Balsa %s", VERSION);
