@@ -104,9 +104,6 @@ static void bmbl_tree_collapse(GtkTreeView * tree_view, GtkTreeIter * iter,
 static gint bmbl_row_compare(GtkTreeModel * model,
                              GtkTreeIter * iter1,
                              GtkTreeIter * iter2, gpointer data);
-static gboolean bmbl_button_press_cb(GtkWidget * widget,
-                                     GdkEventButton * event,
-                                     gpointer data);
 static void bmbl_column_resize(GtkWidget * widget,
                                GtkAllocation * allocation, gpointer data);
 static void bmbl_drag_cb(GtkWidget * widget, GdkDragContext * context,
@@ -130,8 +127,9 @@ static gboolean bmbl_store_redraw_mbnode(GtkTreeIter * iter,
 					 BalsaMailboxNode * mbnode);
 static void bmbl_node_style(GtkTreeModel * model, GtkTreeIter * iter);
 static gint bmbl_core_mailbox(LibBalsaMailbox * mailbox);
-static void bmbl_do_popup(GtkTreeView * tree_view, GtkTreePath * path,
-                          GdkEventButton * event);
+static void bmbl_do_popup(GtkTreeView    *tree_view,
+                          GtkTreePath    *path,
+                          const GdkEvent *event);
 static void bmbl_expand_to_row(BalsaMBList * mblist, GtkTreePath * path);
 /* end of prototypes */
 
@@ -663,28 +661,44 @@ bmbl_row_compare(GtkTreeModel * model, GtkTreeIter * iter1,
     return ret_val;
 }
 
-/* bmbl_button_press_cb:
+/* bmbl_gesture_pressed_cb:
    handle mouse button press events that occur on mailboxes
    (clicking on folders is passed to GtkTreeView and may trigger expand events
 */
-static gboolean
-bmbl_button_press_cb(GtkWidget * widget, GdkEventButton * event,
-                     gpointer data)
+static void
+bmbl_gesture_pressed_cb(GtkGestureMultiPress *multi_press_gesture,
+                        gint                  n_press,
+                        gdouble               x,
+                        gdouble               y,
+                        gpointer              user_data)
 {
-    GtkTreeView *tree_view = GTK_TREE_VIEW(widget);
+    GtkTreeView *tree_view = user_data;
+    GtkGesture *gesture;
+    GdkEventSequence *sequence;
+    const GdkEvent *event;
+    gint bx;
+    gint by;
     GtkTreePath *path;
 
-    if (!gdk_event_triggers_context_menu((GdkEvent *) event)
-        || event->window != gtk_tree_view_get_bin_window(tree_view))
-        return FALSE;
+    gesture  = GTK_GESTURE(multi_press_gesture);
+    sequence = gtk_gesture_single_get_current_sequence(GTK_GESTURE_SINGLE(multi_press_gesture));
+    event    = gtk_gesture_get_last_event(gesture, sequence);
 
-    if (!gtk_tree_view_get_path_at_pos(tree_view, event->x, event->y,
+    if (!gdk_event_triggers_context_menu(event) ||
+        gdk_event_get_window(event) != gtk_tree_view_get_bin_window(tree_view))
+        return;
+
+    gtk_gesture_set_sequence_state(gesture, sequence, GTK_EVENT_SEQUENCE_CLAIMED);
+
+    gtk_tree_view_convert_widget_to_bin_window_coords(tree_view, (gint) x, (gint) y,
+                                                      &bx, &by);
+
+    if (!gtk_tree_view_get_path_at_pos(tree_view, bx, by,
                                        &path, NULL, NULL, NULL))
         path = NULL;
+
     bmbl_do_popup(tree_view, path, event);
     /* bmbl_do_popup frees path */
-
-    return TRUE;
 }
 
 /* bmbl_popup_menu:
@@ -707,14 +721,11 @@ bmbl_popup_menu(GtkWidget * widget)
  * do the popup, and free the path
  */
 static void
-bmbl_do_popup(GtkTreeView * tree_view, GtkTreePath * path,
-              GdkEventButton * event)
+bmbl_do_popup(GtkTreeView    *tree_view,
+              GtkTreePath    *path,
+              const GdkEvent *event)
 {
     BalsaMailboxNode *mbnode = NULL;
-#if !GTK_CHECK_VERSION(3, 22, 0)
-    gint event_button;
-    guint event_time;
-#endif                          /*GTK_CHECK_VERSION(3, 22, 0) */
     GtkWidget *menu;
 
     if (path) {
@@ -726,30 +737,15 @@ bmbl_do_popup(GtkTreeView * tree_view, GtkTreePath * path,
         gtk_tree_path_free(path);
     }
 
-#if !GTK_CHECK_VERSION(3, 22, 0)
-    if (event) {
-        event_button = event->button;
-        event_time = event->time;
-    } else {
-        event_button = 0;
-        event_time = gtk_get_current_event_time();
-    }
-#endif                          /*GTK_CHECK_VERSION(3, 22, 0) */
-
     menu = balsa_mailbox_node_get_context_menu(mbnode);
     g_object_ref(menu);
     g_object_ref_sink(menu);
-#if GTK_CHECK_VERSION(3, 22, 0)
     if (event)
         gtk_menu_popup_at_pointer(GTK_MENU(menu), (GdkEvent *) event);
     else
         gtk_menu_popup_at_widget(GTK_MENU(menu), GTK_WIDGET(tree_view),
                                  GDK_GRAVITY_CENTER, GDK_GRAVITY_CENTER,
                                  NULL);
-#else                           /*GTK_CHECK_VERSION(3, 22, 0) */
-    gtk_menu_popup(GTK_MENU(menu), NULL, NULL, NULL, NULL,
-		   event_button, event_time);
-#endif                          /*GTK_CHECK_VERSION(3, 22, 0) */
     g_object_unref(menu);
 
     if (mbnode)
@@ -871,8 +867,10 @@ bmbl_select_mailbox(GtkTreeSelection * selection, gpointer data)
     GtkTreeModel *model =
         gtk_tree_view_get_model(tree_view);
     GtkTreePath *path;
+    guint button;
+    gdouble x_win, y_win;
 
-    if (!event) {
+    if (event == NULL) {
 	GtkTreeIter iter;
 
 	if (gtk_tree_selection_get_selected(selection, NULL, &iter)) {
@@ -890,20 +888,23 @@ bmbl_select_mailbox(GtkTreeSelection * selection, gpointer data)
 	g_signal_handlers_block_by_func(selection, bmbl_select_mailbox, NULL);
 	gtk_tree_selection_unselect_all(selection);
 	g_signal_handlers_unblock_by_func(selection, bmbl_select_mailbox, NULL);
-        return;
-    }
-    if (event->type != GDK_BUTTON_PRESS
-            /* keyboard navigation */
-        || event->button.button != 1
-            /* soft select */
-        || event->button.window != gtk_tree_view_get_bin_window(tree_view)
-            /* click on a different widget */ ) {
-        gdk_event_free(event);
+
         return;
     }
 
-    if (!gtk_tree_view_get_path_at_pos(tree_view, event->button.x,
-                                       event->button.y, &path,
+    if (gdk_event_get_event_type(event) != GDK_BUTTON_PRESS
+            /* keyboard navigation */
+        || !gdk_event_get_button(event, &button) || button != 1
+            /* soft select */
+        || gdk_event_get_window(event) != gtk_tree_view_get_bin_window(tree_view)
+            /* click on a different widget */ ) {
+        gdk_event_free(event);
+
+        return;
+    }
+
+    if (!gdk_event_get_coords(event, &x_win, &y_win) ||
+        !gtk_tree_view_get_path_at_pos(tree_view, (gint) x_win, (gint) y_win, &path,
                                        NULL, NULL, NULL)) {
         /* GtkTreeView selects the first node in the tree when the
          * widget first gets the focus, whether it's a keyboard event or
@@ -1129,7 +1130,7 @@ bmbl_open_mailbox(LibBalsaMailbox * mailbox, gboolean set_current)
 
     mbnode = balsa_find_mailbox(mailbox);
     if (mbnode == NULL) {
-        g_warning(_("Failed to find mailbox"));
+        g_warning("Failed to find mailbox");
         return;
     }
 
@@ -1178,7 +1179,7 @@ balsa_mblist_close_mailbox(LibBalsaMailbox * mailbox)
 
     mbnode = balsa_find_mailbox(mailbox);
     if (!mbnode)  {
-        g_warning(_("Failed to find mailbox"));
+        g_warning("Failed to find mailbox");
         return;
     }
 
@@ -1253,10 +1254,14 @@ balsa_mblist_close_lru_peer_mbx(BalsaMBList * mblist,
 void
 balsa_mblist_default_signal_bindings(BalsaMBList * mblist)
 {
+    GtkGesture *gesture;
     GtkTreeSelection *selection;
 
-    g_signal_connect(mblist, "button_press_event",
-                     G_CALLBACK(bmbl_button_press_cb), NULL);
+    gesture = gtk_gesture_multi_press_new(GTK_WIDGET(mblist));
+    gtk_gesture_single_set_button(GTK_GESTURE_SINGLE(gesture), 0);
+    g_signal_connect(gesture, "pressed",
+                     G_CALLBACK(bmbl_gesture_pressed_cb), mblist);
+
     g_signal_connect_after(mblist, "size-allocate",
                            G_CALLBACK(bmbl_column_resize), NULL);
     gtk_tree_view_enable_model_drag_dest(GTK_TREE_VIEW(mblist),
@@ -1916,19 +1921,25 @@ bmbl_mru_selected_cb(GtkTreeSelection * selection, gpointer data)
 {
     GdkEvent *event;
     GtkTreeView *tree_view;
+    gdouble x_win, y_win;
     GtkTreePath *path;
 
     if (!data)
         return;
 
     event = gtk_get_current_event();
-    if (!event)
+    if (event == NULL)
         return;
 
+    if (gdk_event_get_event_type(event) != GDK_BUTTON_PRESS ||
+        !gdk_event_get_coords(event, &x_win, &y_win)) {
+        gdk_event_free(event);
+        return;
+    }
+
     tree_view = gtk_tree_selection_get_tree_view(selection);
-    if (event->type != GDK_BUTTON_PRESS ||
-        !gtk_tree_view_get_path_at_pos(tree_view, event->button.x,
-                                       event->button.y, &path,
+
+    if (!gtk_tree_view_get_path_at_pos(tree_view, (gint) x_win, (gint) y_win, &path,
                                        NULL, NULL, NULL)) {
         gtk_tree_selection_unselect_all(selection);
         gdk_event_free(event);
