@@ -91,13 +91,22 @@ static GdkCursor *url_cursor_normal = NULL;
 static GdkCursor *url_cursor_over_url = NULL;
 
 
-static gboolean store_button_coords(GtkWidget * widget, GdkEventButton * event, gpointer data);
-static gboolean check_over_url(GtkWidget * widget, GdkEventMotion * event, gpointer user_data);
+static void store_button_coords(GtkGestureMultiPress *multi_press,
+                                gint                  n_press,
+                                gdouble               x,
+                                gdouble               y,
+                                gpointer              user_data);
+static void check_over_url(BalsaMimeWidgetText * mwt,
+                           message_url_t       * url);
 static void pointer_over_url(GtkWidget * widget, message_url_t * url, gboolean set);
 static void prepare_url_offsets(GtkTextBuffer * buffer, GList * url_list);
 static void url_found_cb(GtkTextBuffer * buffer, GtkTextIter * iter,
                          const gchar * buf, guint len, gpointer data);
-static gboolean check_call_url(GtkWidget * widget, GdkEventButton * event, gpointer user_data);
+static void check_call_url(GtkGestureMultiPress *multi_press,
+                           gint                  n_press,
+                           gdouble               x,
+                           gdouble               y,
+                           gpointer              user_data);
 static message_url_t * find_url(BalsaMimeWidgetText *mwt,
                                 gint                 x,
                                 gint                 y);
@@ -173,6 +182,40 @@ balsa_mime_widget_text_init(BalsaMimeWidgetText * self)
  * End of class definition
  */
 
+/*
+ * Callbacks
+ */
+
+static void
+mwt_controller_motion_cb(GtkEventControllerMotion * motion,
+                         gdouble                    x,
+                         gdouble                    y,
+                         gpointer                   user_data)
+{
+    BalsaMimeWidgetText *mwt = user_data;
+    message_url_t *url;
+
+    url = find_url(mwt, (gint) x, (gint) y);
+    check_over_url(mwt, url);
+}
+
+static void
+mwt_controller_leave_cb(GtkEventControllerMotion * motion,
+                        gdouble                    x,
+                        gdouble                    y,
+                        GdkCrossingMode            mode,
+                        GdkNotifyType              detail,
+                        gpointer                   user_data)
+{
+    BalsaMimeWidgetText *mwt = user_data;
+
+    check_over_url(mwt, NULL);
+}
+
+/*
+ * Public method
+ */
+
 BalsaMimeWidget *
 balsa_mime_widget_new_text(BalsaMessage * bm, LibBalsaMessageBody * mime_body,
 			   const gchar * content_type, gpointer data)
@@ -187,6 +230,7 @@ balsa_mime_widget_new_text(BalsaMessage * bm, LibBalsaMessageBody * mime_body,
     gboolean is_text_plain;
     GtkWidget *widget;
     GtkTextView *text_view;
+    GtkEventController *key_controller;
 
     g_return_val_if_fail(mime_body != NULL, NULL);
     g_return_val_if_fail(content_type != NULL, NULL);
@@ -234,15 +278,8 @@ balsa_mime_widget_new_text(BalsaMessage * bm, LibBalsaMessageBody * mime_body,
 
     /* configure text or source view */
     gtk_text_view_set_editable(text_view, FALSE);
-#if GTK_CHECK_VERSION(3, 23, 1)
     gtk_text_view_set_left_margin(text_view,  BALSA_LEFT_MARGIN);
     gtk_text_view_set_right_margin(text_view, BALSA_RIGHT_MARGIN);
-#else  /* GTK_CHECK_VERSION(3, 23, 1) */
-    gtk_text_view_set_left_margin(text_view, 0);
-    gtk_text_view_set_right_margin(text_view, 0);
-    gtk_widget_set_margin_start(widget, BALSA_LEFT_MARGIN);
-    gtk_widget_set_margin_end(widget, BALSA_RIGHT_MARGIN);
-#endif /* GTK_CHECK_VERSION(3, 23, 1) */
     gtk_text_view_set_wrap_mode(text_view, GTK_WRAP_WORD_CHAR);
 
     /* set the message font */
@@ -265,23 +302,30 @@ balsa_mime_widget_new_text(BalsaMessage * bm, LibBalsaMessageBody * mime_body,
                         is_text_plain);
     g_free(ptr);
 
-    g_signal_connect(widget, "key_press_event",
-                     G_CALLBACK(balsa_mime_widget_key_press_event),
-                    (gpointer) bm);
+    key_controller = gtk_event_controller_key_new(widget);
+    g_signal_connect(key_controller, "key-pressed",
+		     G_CALLBACK(balsa_mime_widget_key_pressed), bm);
+
     g_signal_connect(widget, "populate-popup",
 		     G_CALLBACK(text_view_populate_popup), mwt);
     g_signal_connect_after(widget, "realize",
 			   G_CALLBACK(fix_text_widget), mwt);
 
     if (mwt->url_list != NULL) {
-        g_signal_connect(widget, "button_press_event",
+        GtkGesture *gesture;
+        GtkEventController *motion_controller;
+
+        gesture = gtk_gesture_multi_press_new(widget);
+        g_signal_connect(gesture, "pressed",
                          G_CALLBACK(store_button_coords), NULL);
-        g_signal_connect(widget, "button_release_event",
-                         G_CALLBACK(check_call_url), mwt);
-        g_signal_connect(widget, "motion-notify-event",
-                         G_CALLBACK(check_over_url), mwt);
-        g_signal_connect(widget, "leave-notify-event",
-                         G_CALLBACK(check_over_url), mwt);
+        g_signal_connect(gesture, "released",
+                G_CALLBACK(check_call_url), mwt);
+
+        motion_controller = gtk_event_controller_motion_new(widget);
+        g_signal_connect(motion_controller, "motion",
+                         G_CALLBACK(mwt_controller_motion_cb), mwt);
+        g_signal_connect(motion_controller, "leave",
+                         G_CALLBACK(mwt_controller_leave_cb), mwt);
     }
 
     buffer = gtk_text_view_get_buffer(text_view);
@@ -551,6 +595,8 @@ text_view_url_popup(GtkWidget *widget, GtkMenu *menu, message_url_t *url)
                       G_CALLBACK(url_send_cb), (gpointer)url);
     gtk_menu_shell_append(GTK_MENU_SHELL(menu), menu_item);
 
+    gtk_widget_show_all(GTK_WIDGET(menu));
+
     return TRUE;
 }
 
@@ -598,38 +644,43 @@ text_view_populate_popup(GtkWidget *widget, GtkMenu *menu,
 
 /* -- URL related stuff -- */
 
-static gboolean
-store_button_coords(GtkWidget * widget, GdkEventButton * event, gpointer data)
+static void
+store_button_coords(GtkGestureMultiPress *multi_press,
+                    gint                  n_press,
+                    gdouble               x,
+                    gdouble               y,
+                    gpointer              user_data)
 {
-    if (event->type == GDK_BUTTON_PRESS && event->button == 1) {
-        stored_x = event->x;
-        stored_y = event->y;
-        stored_mask = event->state;
-        /* compare only shift, ctrl, and mod1-mod5 */
-        stored_mask &= STORED_MASK_BITS;
-    }
+    GtkGesture *gesture;
+    const GdkEvent *event;
+    GdkModifierType state;
 
-    return GDK_EVENT_PROPAGATE;
+    gesture = GTK_GESTURE(multi_press);
+    event = gtk_gesture_get_last_event(gesture, gtk_gesture_get_last_updated_sequence(gesture));
+
+    stored_x = (gint) x;
+    stored_y = (gint) y;
+
+    if (gdk_event_get_state(event, &state)) {
+        /* compare only shift, ctrl, and mod1-mod5 */
+        state &= STORED_MASK_BITS;
+        stored_mask = state;
+    }
 }
 
 /* check if we are over an url and change the cursor in this case */
-static gboolean
-check_over_url(GtkWidget * widget, GdkEventMotion * event, gpointer user_data)
+static void
+check_over_url(BalsaMimeWidgetText * mwt,
+               message_url_t       * url)
 {
-    BalsaMimeWidgetText *mwt = user_data;
     static gboolean was_over_url = FALSE;
+    GtkWidget *widget;
     GdkWindow *window;
-    message_url_t *url;
 
+    widget = mwt->text_widget;
     window = gtk_text_view_get_window(GTK_TEXT_VIEW(widget),
                                       GTK_TEXT_WINDOW_TEXT);
 
-    if (event->type == GDK_LEAVE_NOTIFY) {
-        url = NULL;
-    } else {
-        url = find_url(mwt, (gint) event->x, (gint) event->y);
-    }
-
     if (url != NULL) {
         if (url_cursor_normal == NULL || url_cursor_over_url == NULL) {
             GdkDisplay *display;
@@ -655,8 +706,6 @@ check_over_url(GtkWidget * widget, GdkEventMotion * event, gpointer user_data)
     }
 
     mwt->current_url = url;
-
-    return GDK_EVENT_PROPAGATE;
 }
 
 
@@ -732,28 +781,33 @@ url_found_cb(GtkTextBuffer * buffer, GtkTextIter * iter,
 
 /* if the mouse button was released over an URL, and the mouse hasn't
  * moved since the button was pressed, try to call the URL */
-static gboolean
-check_call_url(GtkWidget * widget, GdkEventButton * event, gpointer user_data)
+static void
+check_call_url(GtkGestureMultiPress *multi_press,
+               gint                  n_press,
+               gdouble               x,
+               gdouble               y,
+               gpointer              user_data)
 {
     BalsaMimeWidgetText *mwt = user_data;
-    gint x, y;
+    GtkGesture *gesture;
+    const GdkEvent *event;
+    GdkModifierType state;
 
-    if (event->type != GDK_BUTTON_RELEASE || event->button != 1)
-        return GDK_EVENT_PROPAGATE;
+    gesture = GTK_GESTURE(multi_press);
+    event = gtk_gesture_get_last_event(gesture, gtk_gesture_get_last_updated_sequence(gesture));
+
+    if (event == NULL || !gdk_event_get_state(event, &state))
+        return;
 
-    x = event->x;
-    y = event->y;
     /* 2-pixel motion tolerance */
-    if (abs(x - stored_x) <= 2 && abs(y - stored_y) <= 2
-        && (event->state & STORED_MASK_BITS) == stored_mask) {
+    if (abs((int) x - stored_x) <= 2 && abs((int) y - stored_y) <= 2
+        && (state & STORED_MASK_BITS) == stored_mask) {
         message_url_t *url;
 
         url = find_url(mwt, x, y);
         if (url != NULL)
             handle_url(url->url);
     }
-
-    return GDK_EVENT_PROPAGATE;
 }
 
 /* find_url:
@@ -814,9 +868,6 @@ handle_url(const gchar * url)
         GtkStatusbar *statusbar;
         guint context_id;
         gchar *notice = g_strdup_printf(_("Calling URL %s…"), url);
-#if !GTK_CHECK_VERSION(3, 22, 0)
-        GdkScreen *screen;
-#endif /* GTK_CHECK_VERSION(3, 22, 0) */
         GError *err = NULL;
 
         statusbar = balsa_window_get_statusbar(balsa_app.main_window);
@@ -826,13 +877,8 @@ handle_url(const gchar * url)
         gtk_statusbar_push(statusbar, context_id, notice);
         SCHEDULE_BAR_REFRESH();
         g_free(notice);
-#if GTK_CHECK_VERSION(3, 22, 0)
         gtk_show_uri_on_window(GTK_WINDOW(balsa_app.main_window), url,
                                gtk_get_current_event_time(), &err);
-#else  /* GTK_CHECK_VERSION(3, 22, 0) */
-        screen = gtk_widget_get_screen(GTK_WIDGET(balsa_app.main_window));
-        gtk_show_uri(screen, url, gtk_get_current_event_time(), &err);
-#endif /* GTK_CHECK_VERSION(3, 22, 0) */
 
         if (err != NULL) {
             balsa_information(LIBBALSA_INFORMATION_WARNING,
@@ -1034,50 +1080,50 @@ bm_widget_on_url(const gchar *url)
 
 #ifdef HAVE_HTML_WIDGET
 static void
-bm_zoom_in(BalsaMessage * bm)
+bmwt_html_zoom_in(BalsaMessage * bm)
 {
     balsa_message_zoom(bm, 1);
 }
 
 static void
-bm_zoom_out(BalsaMessage * bm)
+bmwt_html_zoom_out(BalsaMessage * bm)
 {
     balsa_message_zoom(bm, -1);
 }
 
 static void
-bm_zoom_reset(BalsaMessage * bm)
+bmwt_html_zoom_reset(BalsaMessage * bm)
 {
     balsa_message_zoom(bm, 0);
 }
 
 static void
-bm_select_all_cb(GtkWidget * html)
+bmwt_html_select_all_cb(GtkWidget * html)
 {
     libbalsa_html_select_all(html);
 }
 
 static void
-bmwt_populate_popup_menu(BalsaMessage * bm,
-                         GtkWidget    * html,
-                         GtkMenu      * menu)
+bmwt_html_populate_popup_menu(BalsaMessage * bm,
+                              GtkWidget    * html,
+                              GtkMenu      * menu)
 {
     GtkWidget *menuitem;
     gpointer mime_body = g_object_get_data(G_OBJECT(html), "mime-body");
 
     menuitem = gtk_menu_item_new_with_label(_("Zoom In"));
     g_signal_connect_swapped(menuitem, "activate",
-                             G_CALLBACK(bm_zoom_in), bm);
+                             G_CALLBACK(bmwt_html_zoom_in), bm);
     gtk_menu_shell_append(GTK_MENU_SHELL(menu), menuitem);
 
     menuitem = gtk_menu_item_new_with_label(_("Zoom Out"));
     g_signal_connect_swapped(menuitem, "activate",
-                             G_CALLBACK(bm_zoom_out), bm);
+                             G_CALLBACK(bmwt_html_zoom_out), bm);
     gtk_menu_shell_append(GTK_MENU_SHELL(menu), menuitem);
 
     menuitem = gtk_menu_item_new_with_label(_("Zoom 100%"));
     g_signal_connect_swapped(menuitem, "activate",
-                             G_CALLBACK(bm_zoom_reset), bm);
+                             G_CALLBACK(bmwt_html_zoom_reset), bm);
     gtk_menu_shell_append(GTK_MENU_SHELL(menu), menuitem);
 
     menuitem = gtk_separator_menu_item_new();
@@ -1086,7 +1132,7 @@ bmwt_populate_popup_menu(BalsaMessage * bm,
     if (libbalsa_html_can_select(html)) {
         menuitem = gtk_menu_item_new_with_mnemonic(_("Select _All"));
         g_signal_connect_swapped(menuitem, "activate",
-                                 G_CALLBACK(bm_select_all_cb), html);
+                                 G_CALLBACK(bmwt_html_select_all_cb), html);
         gtk_menu_shell_append(GTK_MENU_SHELL(menu), menuitem);
 
         menuitem = gtk_separator_menu_item_new();
@@ -1116,14 +1162,14 @@ bmwt_populate_popup_menu(BalsaMessage * bm,
 }
 
 static gboolean
-balsa_gtk_html_popup(GtkWidget * html, BalsaMessage * bm)
+bmwt_html_popup_context_menu(GtkWidget * html, BalsaMessage * bm)
 {
     GtkWidget *menu;
     const GdkEvent *event;
     GdkEvent *current_event = NULL;
 
     menu = gtk_menu_new();
-    bmwt_populate_popup_menu(bm, html, GTK_MENU(menu));
+    bmwt_html_populate_popup_menu(bm, html, GTK_MENU(menu));
 
     /* In WebKit2, the context menu signal is asynchronous, so the
      * GdkEvent is no longer current; instead it is preserved and passed
@@ -1145,16 +1191,30 @@ balsa_gtk_html_popup(GtkWidget * html, BalsaMessage * bm)
     return TRUE;
 }
 
-static gboolean
-balsa_gtk_html_button_press_cb(GtkWidget * html, GdkEventButton * event,
-                               BalsaMessage * bm)
- {
-    return(gdk_event_triggers_context_menu((GdkEvent *) event)
-            ? balsa_gtk_html_popup(html, bm) : GDK_EVENT_PROPAGATE);
- }
+static void
+bmwt_html_gesture_pressed_cb(GtkGestureMultiPress *multi_press,
+                             gint                  n_press,
+                             gdouble               x,
+                             gdouble               y,
+                             gpointer              user_data)
+{
+    GtkGesture *gesture;
+    const GdkEvent *event;
+    BalsaMessage *bm = user_data;
+
+    gesture = GTK_GESTURE(multi_press);
+    event = gtk_gesture_get_last_event(gesture, gtk_gesture_get_last_updated_sequence(gesture));
+
+    if (gdk_event_triggers_context_menu(event)) {
+        GtkWidget *html;
+
+        html = gtk_event_controller_get_widget(GTK_EVENT_CONTROLLER(gesture));
+        bmwt_html_popup_context_menu(html, bm);
+    }
+}
 
 static void
-bmwt_populate_popup_cb(GtkWidget * widget, GtkMenu * menu, gpointer data)
+bmwt_html_populate_popup_cb(GtkWidget * widget, GtkMenu * menu, gpointer data)
 {
     BalsaMessage *bm =
         g_object_get_data(G_OBJECT(widget), "balsa-message");
@@ -1163,40 +1223,46 @@ bmwt_populate_popup_cb(GtkWidget * widget, GtkMenu * menu, gpointer data)
     /* Remove WebKitWebView's items--they're irrelevant and confusing */
     gtk_container_foreach(GTK_CONTAINER(menu),
                          (GtkCallback) gtk_widget_destroy, NULL);
-    bmwt_populate_popup_menu(bm, html, menu);
+    bmwt_html_populate_popup_menu(bm, html, menu);
 }
 
 static BalsaMimeWidget *
 bm_widget_new_html(BalsaMessage * bm, LibBalsaMessageBody * mime_body)
 {
-    BalsaMimeWidget *mw = g_object_new(BALSA_TYPE_MIME_WIDGET, NULL);
+    BalsaMimeWidgetText *mwt = g_object_new(BALSA_TYPE_MIME_WIDGET_TEXT, NULL);
     GtkWidget *widget;
     GtkWidget *popup_menu;
+    GtkEventController *key_controller;
 
-    widget =
+    mwt->text_widget = widget =
         libbalsa_html_new(mime_body,
                          (LibBalsaHtmlCallback) bm_widget_on_url,
                          (LibBalsaHtmlCallback) handle_url);
-    gtk_container_add(GTK_CONTAINER(mw), widget);
+    gtk_container_add(GTK_CONTAINER(mwt), widget);
 
     g_object_set_data(G_OBJECT(widget), "mime-body", mime_body);
 
-    g_signal_connect(libbalsa_html_get_view_widget(widget),
-                     "key_press_event",
-                     G_CALLBACK(balsa_mime_widget_key_press_event), bm);
+    key_controller = gtk_event_controller_key_new(libbalsa_html_get_view_widget(widget));
+    g_signal_connect(key_controller, "key-pressed",
+		     G_CALLBACK(balsa_mime_widget_key_pressed), bm);
 
     if ((popup_menu = libbalsa_html_popup_menu_widget(widget)) != NULL) {
         g_object_set_data(G_OBJECT(popup_menu), "balsa-message", bm);
         g_signal_connect(popup_menu, "populate-popup",
-                         G_CALLBACK(bmwt_populate_popup_cb), widget);
+                         G_CALLBACK(bmwt_html_populate_popup_cb), widget);
     } else {
-        g_signal_connect(widget, "button-press-event",
-                         G_CALLBACK(balsa_gtk_html_button_press_cb), bm);
+        GtkGesture *gesture;
+
+        gesture = gtk_gesture_multi_press_new(widget);
+        gtk_gesture_single_set_button(GTK_GESTURE_SINGLE(gesture), 0);
+        g_signal_connect(gesture, "pressed",
+                         G_CALLBACK(bmwt_html_gesture_pressed_cb), bm);
+
         g_signal_connect(widget, "popup-menu",
-                         G_CALLBACK(balsa_gtk_html_popup), bm);
+                         G_CALLBACK(bmwt_html_popup_context_menu), bm);
     }
 
-    return mw;
+    return (BalsaMimeWidget *) mwt;
 }
 #endif /* defined HAVE_HTML_WIDGET */
 
@@ -1434,3 +1500,11 @@ fill_text_buf_cited(BalsaMimeWidgetText *mwt,
     if (rex != NULL)
         g_regex_unref(rex);
 }
+
+GtkWidget *
+balsa_mime_widget_text_get_text_widget(BalsaMimeWidgetText *mime_widget_text)
+{
+    g_return_val_if_fail(BALSA_IS_MIME_WIDGET_TEXT(mime_widget_text), NULL);
+
+    return mime_widget_text->text_widget;
+}
