@@ -464,7 +464,7 @@ get_default_user_units(void)
     if (strcmp(e, "default:inch")==0)
 	return GTK_UNIT_INCH;
     else if (strcmp(e, "default:mm"))
-	g_warning("Whoever translated default:mm did so wrongly.\n");
+	g_warning("Whoever translated default:mm did so wrongly.");
     return GTK_UNIT_MM;
 }
 
