@@ -370,7 +370,7 @@ balsa_toolbar_model_append_icon(BalsaToolbarModel * model,
         entry.icon   = g_strdup(real_button);
         g_array_append_val(model->current, entry);
     } else
-        g_warning(_("Unknown toolbar icon “%s”"), icon);
+        g_warning("Unknown toolbar icon “%s”", icon);
 }
 
 /* Remove all icons from the BalsaToolbarModel.
@@ -623,43 +623,6 @@ tm_remove_underscore(const gchar * text)
     return r;
 }
 
-#if !GTK_CHECK_VERSION(3, 22, 0)
-/* If the menu is popped up in response to a keystroke, center it
- * immediately below the toolbar.
- */
-static void
-tm_popup_position_func(GtkMenu * menu, gint * x, gint * y,
-                       gboolean * push_in, gpointer user_data)
-{
-    GtkWidget *toolbar = GTK_WIDGET(user_data);
-    GdkScreen *screen = gtk_widget_get_screen(toolbar);
-    GtkRequisition req;
-    gint monitor_num;
-    GdkRectangle monitor;
-    GtkAllocation allocation;
-
-    g_return_if_fail(gtk_widget_get_window(toolbar));
-
-    gdk_window_get_origin(gtk_widget_get_window(toolbar), x, y);
-
-    gtk_widget_get_preferred_size(GTK_WIDGET(menu), NULL, &req);
-
-    gtk_widget_get_allocation(toolbar, &allocation);
-    *x += (allocation.width - req.width) / 2;
-    *y += allocation.height;
-
-    monitor_num = gdk_screen_get_monitor_at_point(screen, *x, *y);
-    gtk_menu_set_monitor(menu, monitor_num);
-    gdk_screen_get_monitor_geometry(screen, monitor_num, &monitor);
-
-    *x = CLAMP(*x, monitor.x,
-               monitor.x + MAX(0, monitor.width - req.width));
-    *y = CLAMP(*y, monitor.y,
-               monitor.y + MAX(0, monitor.height - req.height));
-
-    *push_in = FALSE;
-}
-#endif                          /*GTK_CHECK_VERSION(3, 22, 0) */
 
 static gboolean
 tm_popup_context_menu_cb(GtkWidget    * toolbar,
@@ -669,9 +632,6 @@ tm_popup_context_menu_cb(GtkWidget    * toolbar,
                          toolbar_info * info)
 {
     GtkWidget *menu;
-#if !GTK_CHECK_VERSION(3, 22, 0)
-    int event_time;
-#endif                          /*GTK_CHECK_VERSION(3, 22, 0) */
     guint i;
     GSList *group = NULL;
     GtkToolbarStyle default_style;
@@ -766,8 +726,7 @@ tm_popup_context_menu_cb(GtkWidget    * toolbar,
     gtk_menu_attach_to_widget(GTK_MENU(menu), toolbar, NULL);
 
     event = gtk_get_current_event();
-#if GTK_CHECK_VERSION(3, 22, 0)
-    if (event != NULL && event->type == GDK_BUTTON_PRESS) {
+    if (event != NULL && gdk_event_get_event_type(event) == GDK_BUTTON_PRESS) {
         gtk_menu_popup_at_pointer(GTK_MENU(menu), event);
     } else {
         gtk_menu_popup_at_widget(GTK_MENU(menu),
@@ -776,16 +735,6 @@ tm_popup_context_menu_cb(GtkWidget    * toolbar,
                                  GDK_GRAVITY_SOUTH,
                                  NULL);
     }
-#else                           /*GTK_CHECK_VERSION(3, 22, 0) */
-    event_time = gtk_get_current_event_time();
-
-    if (button >= 0)
-        gtk_menu_popup(GTK_MENU(menu), NULL, NULL, NULL, NULL, button,
-                       event_time);
-    else
-        gtk_menu_popup(GTK_MENU(menu), NULL, NULL, tm_popup_position_func,
-                       toolbar, button, event_time);
-#endif                          /*GTK_CHECK_VERSION(3, 22, 0) */
 
     if (event != NULL)
         gdk_event_free(event);
